<?php

/**
 * class mailer
 *
 * Class to generate a mail with html AND plain text message.
 * Also possible to attach files
 *
 * Methods:
 * ---------------------------------------------------------------------
 * mailer()                       - Constructor, set some default values
 * setHTMLMessage( $message )     - Set the HTML message
 * setMessage( $message )         - Set a plain text message
 * attachFile( $file [, $mime] )  - Attach a file to the mail
 * includeFile( $file [, $mime] ) - Attach a file which can be used in the HTML message. The name which can be used in the HTML will be returned
 * setParameters( $paramenters )  - Set extra parameters for the mail function
 * setBoundary( $boundary )       - Set a specific boundary
 * setFrom( $mail [, $name] )     - Set the from e-mail adres and name
 * setReplyTo( $mail [, $name] )  - Set the reply-to e-mail adres and name
 * setCc( $cc )                   - Set the cc addresses to send a copy of a mail to
 * setBcc( $bcc )                 - Set the bcc addresses to send a "blind" copy of the mail
 * setHeader( $header )           - Set extra headers to the mail 
 * send( $to, $subject )          - Send the mail
 *
 * Example:
 * ----------------------------------------------------------------------
 * $m = new mailer();
 * 
 * $img = $m->includeFile( "logo.gif", "image/gif" );
 * $m->setHTMLMessage( "This is a <b>test</b> e-mail <i>with</i> HTML! <img src=\"$img\" />" );
 * $m->setMessage("This is a plain text message...");
 *
 * $m->send("john@doe.com", "Test Mail!" );
 * 
 * @author Teye Heimans
 */
class mailer {
    var $_headers;
    var $_boundary; 
    var $_message;    
    var $_params;

    /**
     * mailer::mailer()
     *
     * Constructor, set some default values.
     *
     * @access protected
     * @author Teye Heimans   
     */
    function mailer() 
    {
        // always use \r\n in your mail and to split the headers!
        if( !defined('NL') ) {
            define('NL', "\r\n");
        }
        
        // create a unique boundary
        $this->setBoundary( uniqid()."x|x".uniqid() );
        
        // Set the default text message
        // this will be overwritten if the user set's another 
        // plain text message
        $this->setMessage(
          'If you are reading this, it means that your e-mail client'.NL.
          'does not support MIME. Please upgrade your e-mail client'.NL.
          'to one that supports MIME.' 
        );
        
        // set the from adres when it can be retreived from the php.ini
        $from = ini_get("sendmail_from");
        if( !empty($from) ) {
            $this->setFrom( $from );
        }
    }
    
    /**
     * mailer::setHTMLMessage()
     *
     * Set the HTML message. Most clients do not accept HTML e-mails, so
     * you should ALWAYS set a plain text message!
     *
     * @param string $message: The HTML message to set
     * @return void
     * @access public
     * @author Teye Heimans   
     */
    function setHTMLMessage( $message ) 
    {
        $this->_message['html'] = $message;        
    }
    
    /**
     * mailer::setMessage()
     *
     * Set the plain text message. Most clients do not accept HTML e-mails, so
     * you should ALWAYS set a plain text message!
     *
     * @param string $message: The message to set
     * @return void
     * @access public
     * @author Teye Heimans   
     */
    function setMessage( $message ) 
    {
        $this->_message['plain'] = $message; 
    }
    
    
    /**
     * mailer::attachFile()
     *
     * Attach a file to the mail
     *
     * @param string $file: the file we have to attach to the mail
     * @param string $mime: if the mime type cant be retrieved you have to spicify the mime type of the file
     * @return void
     * @access public
     * @author Teye Heimans   
     */
    function attachFile( $file, $mime = null )  
    {
        // retieve the mime type of the file if possible
        if( function_exists('mime_content_type') ) 
        {
            $mime = mime_content_type( $file );
            
        }
        // if the mime type can not be retrieved and it is not given,
        // display an error message
        if ( is_null( $mime) || !$mime ) 
        {
            trigger_error(
              'The mime type of the file "'.$file.'" could not be retrieved.' . NL .
              'Please set the mime type of the file as second argument in the method attachFile().' . NL .
              'If you don\'t know the mime type, you can retireve it at http://filext.com.',
              E_USER_WARNING
            );
            return;   
        }
        
        // check if the file exists and if it is readable and open it
        if( !file_exists( $file ) || !is_readable( $file ) || !($fd = fopen($file, 'rb')) ) 
        {
            trigger_error(
              'The file "'.$file.'" could not be read. Make sure that the '.
              'file exists and that is readable.',
              E_USER_WARNING
            );
            return;        
        }
        
        // read the data from the file
        $data = fread( $fd, filesize($file) );
        fclose( $fd );
        
        // get the name of the file
        $name = basename($file);
        
        // create the attachment data    
        $this->_message['attachment'][] = 
          'Content-Type: '.$mime.'; name="'.$name.'"' . NL .
          'Content-Transfer-Encoding: base64' . NL .
          'Content-disposition: attachment; file="'.$name.'"' . NL . NL .
          chunk_split( base64_encode( $data ) ) . NL ;
    }
    
    /**
     * mailer::includeFile()
     *
     * HTML emails will attempt to download their images and style sheets from the web.
     * Because of security and privacy reasons, many clients will refuse to attempt these downloads,
     * ruining the look of your HTML message. With this function you can attach the files 
     * to your e-mail and point to them in your HTML.
     *
     * @param string $file: the file we have to attach to the mail
     * @param string $mime: if the mime type cant be retrieved you have to spicify the mime type of the file
     * @return string: the name you can use in your HTML tag, like <img src="xxx" />
     * @access public
     * @author Teye Heimans   
     */
    function includeFile( $file, $mime ) 
    {
        // retieve the mime type of the file if possible
        if( function_exists('mime_content_type') ) 
        {
            $mime = mime_content_type( $file );
            
        }
        // if the mime type can not be retrieved and it is not given,
        // display an error message
        if ( is_null( $mime) || !$mime ) 
        {
            trigger_error(
              'The mime type of the file "'.$file.'" could not be retrieved.' . NL .
              'Please set the mime type of the file as second argument in the method attachFile().' . NL .
              'If you don\'t know the mime type, you can retireve it at http://filext.com.',
              E_USER_WARNING
            );
            return;   
        }
        
        // check if the file exists and if it is readable and open it
        if( !file_exists( $file ) || !is_readable( $file ) || !($fd = fopen($file, 'rb')) ) 
        {
            trigger_error(
              'The file "'.$file.'" could not be read. Make sure that the '.
              'file exists and that is readable.',
              E_USER_WARNING
            );
            return;        
        }
        

        // read the data from the file
        $data = fread( $fd, filesize($file) );
        fclose( $fd );
        
        // get the name of the file
        $name = basename($file);
        
        // create the attachment data    
        $this->_message['include'][] = 
          'Content-Type: '.$mime.'; name="'.$name.'"' . NL .
          'Content-Transfer-Encoding: base64' . NL .
          'Content-Location: '.$name . NL . NL .
          chunk_split( base64_encode( $data ) ) . NL ;
          
        return $name;      
    }
    
    /**
     * mailer::setParameters()
     *
     * Set additional parameters for the mail function
     *
     * @param string $params: the additional params
     * @return void
     * @access public
     * @author Teye Heimans   
     */
    function setParameters( $params ) 
    {
        $this->_params = $params;
    } 
    
    /**
     * mailer::send()
     *
     * Create the mail and send it
     *
     * @param $to: The e-mail adres where we should send the mail to
     * @param $subject: The subject of the mail
     * @return bool
     * @access public
     * @author Teye Heimans   
     */
    function send( $to, $subject ) 
    {
        // is the mail a mime mail?
        $mime = false;
        
        // does the mail contains included attachemnts?
        $includes = false;
        
        // set the plain text body
        $body = $this->_message['plain'] . NL;
        
        // is there a html message ?
        if( isset( $this->_message['html'] ) ) 
        {
            // the mail is a mime mail..
            $mime = true;            
            
            // set the html body
            $body .= 
            '--' . $this->_boundary . NL .
            'Content-Type: text/html; charset="iso-8859-1"' . NL .
            'Content-Transfer-Encoding: 7bit' . NL . NL .
            $this->_message['html'] . NL;
        }
        
        // are there attachments ?
        if( isset( $this->_message['attachment'] ) ) 
        {
            foreach( $this->_message['attachment'] as $data ) {
                $body .= '--' . $this->_boundary . NL . $data;
            }
        }
        
        // are there attachments ?
        if( isset( $this->_message['include'] ) ) 
        {
            $includes = true;            
            foreach( $this->_message['include'] as $data ) {
                $body .= '--' . $this->_boundary . NL . $data;
            }
            
        }
        
        // if it's a mime mail, set the end boundary
        if( $mime ) 
        {
            $body .= '--' . $this->_boundary . '--' . NL;
            
            // set the mime headers if it's a mime mail
            $this->setHeader( 
              'MIME-Version: 1.0' . NL .
              'Content-Type: multipart/'.($includes ? 'related':'mixed').'; boundary="'.$this->_boundary.'"' . NL 
            );
        }        
        
        // send the mail!
        return mail(
          $to, 
          $subject,
          $body,
          $this->_headers,
          $this->_params
        );  
    }
    
    /**
     * mailer::setBoundary()
     *
     * Set the boundary which is used to split the message in blocks. The
     * boundary has to be a unique string which will not occur in the messages!
     * Only use this function if you know what you are doing. 
     * The boundary which is set by default works fine.
     *
     * @param string $boundary: The new boundary
     * @return void
     * @access public
     * @author Teye Heimans   
     */
    function setBoundary( $boundary ) 
    {
        $this->_boundary = $boundary;
    }
        
    
    /**
     * mailer::setFrom()
     *
     * Set the "From" adres of the mail
     *
     * @param string $email: The e-mail adres of the sender
     * @param string $name: The name of the sender
     * @return void
     * @access public
     * @author Teye Heimans   
     */
    function setFrom( $email, $name = '' ) 
    {        
        if( empty( $name ) ) {
            $name = $email;
        }
        $this->_headers .= 'From: '.$name.' <'.$email.'>'.NL;
    }
    
    /**
     * mailer::setReplyTo()
     *
     * Set the "reply-to" adres for the mail
     *
     * @param string $email: The e-mail adres where the receiver should reply to
     * @param string $name: The name of the sender
     * @return void
     * @access public
     * @author Teye Heimans   
     */
    function setReplyTo( $email, $name = '' ) 
    {
        if( empty( $name ) ) {
            $name = $email;
        }
        $this->_headers .= 'Reply-To: '.$name.' <'.$email.'>'.NL;
    }
    
    /**
     * mailer::setCc()
     *
     * Send a "carbon copy" to the given email addresses.
     * Split multiple e-mail addresses with a space or a comma.
     * (CC = a copy of the mail, the receiver can see to who the mail is also send)
     *
     * @param string $cc: The e-mail address(es) of the persons who should recieve a copy of the mail
     * @return void
     * @access public
     * @author Teye Heimans   
     */
    function setCc( $cc ) 
    {
        $this->_headers .= 'Cc: '.$name.' <'.$email.'>'.NL;
    }
    
    /**
     * mailer::setBcc()
     *
     * Send a "blind carbon copy" to the given email addresses.
     * Split multiple e-mail addresses with a space or a comma.
     * (CC = a copy of the mail, the receiver can NOT see to who the mail is also send)
     *
     * @param string $bcc: The e-mail address(es) of the persons who should recieve a copy of the mail
     * @return void
     * @access public
     * @author Teye Heimans   
     */
    function setBcc( $bcc ) 
    {
        if( phpversion() < 4.3 && strpos( "win", strtolower($_SERVER['SERVER_SOFTWARE'])) !== false ) 
        {
            trigger_error(
              'Error, could not set the bcc addresses. The PHP version does not support this!',
              E_USER_WARNING
            );
            return; 
        }
        
        $this->_headers .= 'Bcc: '.$name.' <'.$email.'>'.NL;
    }
    
    /**
     * mailer::setHeader()
     *
     * Add an extra header to the mail
     *
     * @param string $header: The extra header you want to add
     * @return void
     * @access public
     * @author Teye Heimans   
     */
    function setHeader( $header ) 
    {
        // add new line to the header if it does not contains it.
        if( !preg_match("/\r|\n/", $header) ) {
            $header .= NL;
        }
        $this->_headers .= $header;
    }
}

?>